#!/usr/bin/perl
use strict;
use IO::Socket::INET;
use Getopt::Long;
my ($opt_help, $opt_host, $opt_port);
GetOptions('h'			=> \$opt_help,
			'host=s'	=> \$opt_host,
			'port=i'	=> \$opt_port) || &usage();

&usage() if $opt_help;

my $EOL = "\015\012";
my $host = $opt_host || "localhost";
my $port = $opt_port || die(&usage);

my $sock = new IO::Socket::INET->new(
							Proto	=> 'tcp',
							PeerAddr => $host,
							PeerPort => $port)
			or die("Error connecting to port $port on host $host: $!\n");
my $numAttempts;
my $auth;
my $prompt;
while (not($auth) and ($numAttempts < 3)) {
	#Get the prompt from the server.
	$prompt = <$sock>;
	#remove the \r\n from the incoming stream.
	chop $prompt;
	chop $prompt;
	print $prompt . "\n";
	$numAttempts++;
	my ($login, $password);
	if ($prompt eq "Login:") {
		# Read the login name from the stdin
 		$login = <STDIN>;
		chomp $login;
	} else {
		die("Check yourself, probably you are not following the protocol\n");
	}

	# Send the data over the socket.
	print $sock $login . $EOL;
	
	# Get the Password: prompt from the server.
	$prompt = <$sock>;

	#remove the \r\n from the incoming stream.
	chop $prompt;
	chop $prompt;

	print $prompt . "\n";
	if ($prompt eq "Password:") {
		`stty -echo`;
		$password = <STDIN>;
		`stty echo`;
		chomp $password;
	} else {
		die("Check yourself, probably you are not following the protocol\n");
	}
	print $sock $password. $EOL;
	# Get the servers response. If its Login Incorrect, go back to while, else continue processing.
	$prompt = <$sock>;
	chop $prompt;
	chop $prompt;
	print $prompt . "\n";
	next if ($prompt eq "Login Incorrect");
	$auth=1;
}

if (($numAttempts == 3) and not $auth) {
	close $sock;
	exit (1);
}

# We reach here if successfully authenticated.

# Get the prompt
# server asks for exam number.
$prompt = <$sock>;
while (1) {
	chop $prompt;
	chop $prompt;
	print $prompt . "\n";

	#Read from stdin.
	# client reads exam number from stdin.
	my $found=1;
	my $response;
	while ($found) {
		$response = <STDIN>;
		chomp $response;
		if ($response !~ /^\d+$/) {
			print STDOUT "Invalid exam number, enter new exam number\n";
		} else {
			$found = 0;
		}
	}
	# writes exam number to socket.
	print $sock $response. $EOL;

	# server responds with marks/ says invalid exam number if thats the case.
	$prompt = <$sock>;
	print $prompt;
	chop $prompt;
	chop $prompt;
	if (not($prompt eq "Invalid exam number")) {
		# server sends the average marks.
		$prompt =  <$sock>;
		print $prompt;
	}
	# Do you want to get marks for other subjects?
	$prompt = <$sock>;
	print $prompt;
	# Get y/n fom user.
	$response = <STDIN>;
	chomp $response;
	# write to socket.
	print $sock $response. $EOL;
	$prompt =  <$sock>;
	# if y, server returns "Which exam?", n server closes connection.
	if (not($prompt)) {
		# Server closed the connection on you. You can close too.
		close $sock;
		exit;
	}
	# if something meaningful came on the last read then continue to while.
}

#####################################################
# SUBROUTINES
#####################################################
sub usage {
	my $helpMessage = 
	"Usage: client.pl [-h] [-host server host] -port server port
	Where:
	-h : Display this help message
	-host: Host on which the server is running. [default=localhost]
	-port: TCP port on which the server is listening for requests\n";
	print $helpMessage;
	exit;
}
