#!/usr/bin/perl
use strict;
use Socket;
use IO::Handle;
use POSIX ":sys_wait_h";
use Getopt::Long;

my ($opt_help, $opt_port, $opt_queue, $opt_pass, $opt_marks, $opt_log);

GetOptions(	'h'		=> \$opt_help,
       	'port=i'	=> \$opt_port,
		'queue=i'	=> \$opt_queue,
		'pass=s'	=> \$opt_pass,
		'marks=s'	=> \$opt_marks,
		'log=s'		=> \$opt_log) || &usage();

if ($opt_help) {&usage();}

my $port = $opt_port || die(&usage);
my $logFile = $opt_log || "server.log";
my $queueSize = $opt_queue || 10;#The maximum size of the queue of waiting connections.
my $passfile = $opt_pass || "pass.txt";
if (not (-r $passfile)) {print "Cannot read $passfile\n"; exit;} 
my $marksfile = $opt_marks || "exam.txt"; 
if (not (-r $marksfile)) {print "Cannot read $marksfile\n"; exit;}
my $EOL = "\015\012";

my $waitPid;
# open the log file.
# create TCP socket for this server to listen on.
my $servSock;
my $proto = getprotobyname('tcp');
socket($servSock, PF_INET, SOCK_STREAM, $proto) || die("Error creating a socket: $!\n");

# Avoid the address in use error.
setsockopt($servSock, SOL_SOCKET, SO_REUSEADDR, 1) || die("Error in setsockopt: $!\n");

#Bind to a port.
my $packedAddr = pack("Sna4x8" , AF_INET, $port, INADDR_ANY);
bind($servSock, $packedAddr) || die("Error in bind: $!\n");
undef $packedAddr;

# listen at the specified port for new requests.
listen($servSock, $queueSize) || writeLog("Error listening on socket\n");

writeLog("Server started on port $port\n");

# Set a signal handler for SIGCHLD. We are trying to avoid zombie child processes from hanging 
# around in the system.
$SIG{CHLD} = \&CHLDHDLR;

my $packedAddr;
while (1) {
	my $clientSock;
	my $childPid;
	if (($packedAddr = accept($clientSock, $servSock)) || $waitPid) {
 		if ($waitPid  and not $packedAddr) {$waitPid = 0; next; }
		$waitPid = 0;
		# Spawn off a child process for each new incoming request.
		if ($childPid = fork()) {
			# This is the parent process
			# Obtain the address of the client from which this request was made.
			my ($remotePort, $remoteAddr) = sockaddr_in($packedAddr);
			close($clientSock);
			writeLog("Connection from " . inet_ntoa($remoteAddr). " port " . $remotePort . "\n");
			# Go back to the while loop and accept connections.
		} else {
			# This is the child process.
			# Call the subroutine to do the processing.
			close($servSock); # The child does not need this anyway.
			&processRequest($clientSock);
			close($clientSock);
			exit;
		}
	}
}

############################################################
#   SUBROUTINES
############################################################
sub usage {
my $helpMessage = 
"Usage: server.pl [-h] -port Portnumber [-queue queueSize] [-pass passwordfile] [-marks marksfile] [-log logfile]
Where:
-h	: Display this usage message.
-port	: TCP port at which the server will listen [compulsory argument]
-queue	: Maximum size of the queue of outstanding requests [default=10]
-pass	: password file path [default=pass.txt]
-marks	: marks file path [default=exam.txt]
-log 	: log file [default = server.log]\n";
print $helpMessage;
exit(1);
}

sub processRequest {
	my $socket = shift;
	# Flush after each line is written to the socket.
	$socket->autoflush(1);
	my $loginAttempts;
 LOOP: while ($loginAttempts < 3) {
 	$loginAttempts++;
	print $socket "Login:" . $EOL;
	my $username = <$socket>;
	chop $username;
	chop $username;
	print $socket "Password:" . $EOL;
	my $password = <$socket>;
	chop $password;
	chop $password;
# open the password file.
	open PASSFILE, $passfile;
	my $found;
	while (not($found) and (my $record = <PASSFILE>)) {
	 chomp $record;
	 my ($field1, $field2) = split(/[\s\t]+/, $record);
	 if (($field1 eq $username) and ($field2 eq $password)) {
		 $found = 1;
	 }
	}
	if ($found) {
		$loginAttempts = 3;  
		print $socket "User authenticated" . $EOL;
  		close PASSFILE;
		my $response;
		while (($response eq 'y') || ($response eq 'yes') || not ($response)) {
			print $socket "Enter exam number" . $EOL;
			my $examNo = <$socket>;
			chop $examNo;
			chop $examNo;
			&getMarks($socket, $username, $examNo);
			print $socket "Do you want to know marks in other subjects? Respond with y/n" . $EOL;
			$response = <$socket>;
			chop $response;
			chop $response;
		}
	} else {
		print $socket "Login Incorrect". $EOL;
  		close PASSFILE;
		next LOOP;
	}
  }
	return;
}

sub writeLog {
	open(LOGFILE, ">>$logFile") || die("Cannot open log file: $!");
	print LOGFILE shift;
	close LOGFILE;
}

sub CHLDHDLR {
	# WNOHANG below means dont wait if there is no child to be cleaned up.
	while (($waitPid = waitpid(-1, WNOHANG)) > 0) {
#		writeLog("child process with pid $pid ended\n");
	}
	#reinstall the signal handler to avoid problems if the handler is set to SIG_DFL
	$SIG{CHLD} = \&CHLDHDLR;
}

sub getMarks {
	my ($socket, $user, $exam) = @_;
	# open the marks file.
	open (MARKS, $marksfile) || die("Error opening the marks file");
	my $marks;
	my $avgMarks;
	my $numStudents;
	while (<MARKS>) {
			chomp;
			my @markList = split /[\s\t]+/, $_;
			if (@markList < $exam+1) {
				print $socket "Invalid exam number" . $EOL;
				return;
			}
			if (($markList[0] eq $user) and not($marks)) {
				$marks = $markList[$exam];
			} 	
			$avgMarks = $avgMarks + $markList[$exam];
			$numStudents++;
	}
	$avgMarks = $avgMarks/$numStudents;
	print $socket "Your marks in Exam $exam = $marks" . $EOL;
	print $socket "Avg. class marks in Exam $exam = $avgMarks" . $EOL;
	close MARKS;
	return;
}	
